local frame, events = CreateFrame("Frame"), {}
local WL_ChatAnswer = "|cffffcc00Worthless:|r "
local isAdiBags = false
local isBaggins = false
local isBagnon = false
local isCombuctor = false
local isInventorian = false
local isTdBag2 = false

local qualities = {
	["0"] = "|cff9d9d9dPoor|r",
	["1"] = "|cffe8e8e8Common|r",
	["2"] = "|cff1eff00Uncommon|r",
	["3"] = "|cff0070ffRare|r",
	["4"] = "|cffa335eeEpic|r",
	["5"] = "|cffff8000Legendary|r",
}

local WaitTable = {}
local WaitFrame = nil

local function Wait(delay, func, ...)
	if(type(delay) ~= "number" or type(func) ~= "function") then
		return false
	end

	if(WaitFrame == nil) then
		WaitFrame = CreateFrame("Frame", "WaitFrame", UIParent)
		WaitFrame:SetScript("onUpdate", function (self, elapse)
			local count = #WaitTable
			local i = 1

			while(i <= count) do
				local waitRecord = tremove(WaitTable, i)
				local d = tremove(waitRecord,1)
				local f = tremove(waitRecord,1)
				local p = tremove(waitRecord,1)

				if(d > elapse) then
					tinsert(WaitTable,i,{d-elapse,f,p})
					i = i + 1
				else
					count = count - 1
					f(unpack(p))
				end
			end
		end)
	end

	tinsert(WaitTable, {delay, func, {...}})

	return true;
end

--[[
	Finds out if the player's inventory still has some free slots.

	@return boolean
--]]
local function HasFreeInventorySlots()
	local freeSlots = 0

	for i = 0, 4 do
		freeSlots = freeSlots + GetContainerNumFreeSlots(i)
	end

	return freeSlots > 0
end

--[[
	Since I couldn't find any other method of finding out when bags are opened having AdiBags installed,
	this function searches for AdiBagsContainer1 which is created at runtime when opening the player's
	inventory for the first time.
--]]
local function SearchForAdiBagsContainer()
	local AdiBagsSearchFrame = CreateFrame("Frame")

	local function WL_ON_UPDATE(self, elapsed)
		self.TimeSinceLastUpdate = (self.TimeSinceLastUpdate or 0) + elapsed

		if(self.TimeSinceLastUpdate > 1) then
			if(AdiBagsContainer1 ~= nil) then
				WL_MarkItemsAsCrap()
				AdiBagsSearchFrame:SetScript("OnUpdate", nil)
			end

			self.TimeSinceLastUpdate = 0
		end
	end

	AdiBagsSearchFrame:SetScript("OnUpdate", WL_ON_UPDATE)
end

--[[
	Since I couldn't find any other method of finding out when bags are opened having tdBag2 installed,
	this function searches for tdBag2MoneyFrame1 which is created at runtime when opening the player's
	inventory for the first time.
--]]
local function SearchForTdBag2Container()
	local TdBag2SearchFrame = CreateFrame("Frame")

	local function WL_ON_UPDATE(self, elapsed)
		self.TimeSinceLastUpdate = (self.TimeSinceLastUpdate or 0) + elapsed

		if(self.TimeSinceLastUpdate > 1) then
			if(tdBag2MoneyFrame1 ~= nil) then
				WL_MarkItemsAsCrap()
				TdBag2SearchFrame:SetScript("OnUpdate", nil)
			end

			self.TimeSinceLastUpdate = 0
		end
	end

	TdBag2SearchFrame:SetScript("OnUpdate", WL_ON_UPDATE)
end

--[[
	Init the database.
--]]
function events:ADDON_LOADED()
	if(WLDB == nil) then
		WLDB = {}
	end

	if(WLConfig == nil) then
		WLConfig = {}
	end

	if(WLConfig.qualityThreshold == nil) then
		WLConfig.qualityThreshold = 0
	end

	if(WLConfig.bordertype == nil) then
		WLConfig.bordertype = "shine"
	end

	if(WLConfig.borderColourR == nil) then
		WLConfig.borderColourR = 170
	end

	if(WLConfig.borderColourG == nil) then
		WLConfig.borderColourG = 45
	end

	if(WLConfig.borderColourB == nil) then
		WLConfig.borderColourB = 15
	end

	if(WLConfig.fullMode == nil) then
		WLConfig.fullMode = false
	end
end

--[[
	Choose actions depending on installed addons.
]]--
function events:PLAYER_LOGIN(...)	
	if(select(4, GetAddOnInfo("AdiBags")) == true) then
		isAdiBags = true
		SearchForAdiBagsContainer()
		print(WL_ChatAnswer .. "AdiBags detected.")
	elseif(select(4, GetAddOnInfo("Baggins")) == true) then
		isBaggins = true
		BagginsBag1:HookScript("OnShow", function() Wait(0.1, WL_MarkItemsAsCrap) end)
		print(WL_ChatAnswer .. "Baggins detected.")
	elseif(select(4, GetAddOnInfo("Bagnon")) == true) then
		isBagnon = true
		BagnonFrameinventory:HookScript("OnShow", function() Wait(0.1, WL_MarkItemsAsCrap) end)
		print(WL_ChatAnswer .. "Bagnon detected.")
	elseif(select(4, GetAddOnInfo("Combuctor")) == true) then
		isCombuctor = true
		CombuctorFrameinventory:HookScript("OnShow", function() Wait(0.1, WL_MarkItemsAsCrap) end)
		print(WL_ChatAnswer .. "Combuctor detected.")
	elseif(select(4, GetAddOnInfo("Inventorian")) == true) then
		isInventorian = true
		InventorianBagFrame:HookScript("OnShow", WL_MarkItemsAsCrap)
		print(WL_ChatAnswer .. "Inventorian detected.")
	elseif(select(4, GetAddOnInfo("tdBag2")) == true) then
		isTdBag2 = true
		SearchForTdBag2Container()
		print(WL_ChatAnswer .. "tdBag2 detected.")
	else
		hooksecurefunc("OpenBackpack", WL_MarkItemsAsCrap)
		hooksecurefunc("OpenBag", WL_MarkItemsAsCrap)
		print(WL_ChatAnswer .. "No bag addon detected.")
	end

	-- Different slash commands in case of Wowhead Looter installed (which also uses /wl)
	SLASH_WL1 = '/worthless'

	if(select(4, GetAddOnInfo("Wowhead_Looter")) == true) then
		SLASH_WL2 = '/wls'
	else
		SLASH_WL2 = '/wl'
	end
end

function events:PLAYER_ENTERING_WORLD(...)
	WL_MarkItemsAsCrap()
end

function events:BAG_UPDATE(...)
	WL_MarkItemsAsCrap()
end

function events:ITEM_PUSH(...)
	WL_MarkItemsAsCrap()
end

function events:MERCHANT_SHOW(...)
	WL_MarkItemsAsCrap()
end

function events:MERCHANT_CLOSED(...)
	WL_MarkItemsAsCrap()
end

function events:MERCHANT_UPDATE(...)
	WL_MarkItemsAsCrap()
end

frame:SetScript("OnEvent", function(self, event, ...)
	events[event](self, ...)
end)

for k, v in pairs(events) do
	frame:RegisterEvent(k)
end

--[[
	Extracts item information from an item link.

	@param itemLink

	@return The item id
]]--
local function GetItemInfoFromLink(itemLink)
	local _, _, color, lType, id, enchant, gem1, gem2, gem3, gem4, suffix, unique, linkLvl, name = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*):?(%-?%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")

	return id
end

--[[
	Creates a border ready to be put onto an icon.
]]--
function CreateBorder(name, parent)
	local border = parent:CreateTexture("WLBorderTexture", "OVERLAY")

	border:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	border:SetBlendMode("ADD")
	border:SetAlpha(1)
	border:SetHeight(64)
	border:SetWidth(64)
	border:SetPoint("CENTER", parent, "CENTER")

	return border
end

--[[
	Applies a border to an icon and sets its colour.
]]--
local function ApplyCustomBorder(frame)
	frame.wlborder = CreateBorder("WLBorder", frame)
	frame.wlborder:SetVertexColor(WLConfig.borderColourR / 255, WLConfig.borderColourG / 255, WLConfig.borderColourB / 255)
	frame.wlborder:SetAlpha(1)
end

local function MarkBaggins()
	local crappiestItemSlot, crappiestItemSellPrice, crappiestItemQuality

	for i = 1, 1000 do
		-- Check if the BagginsPooledItemButton is in use
		if(_G["BagginsPooledItemButton" .. i] ~= nil and _G["BagginsPooledItemButton" .. i]:GetParent() ~= nil) then
			local id = _G["BagginsPooledItemButton" .. i]:GetID()
			local parentId = _G["BagginsPooledItemButton" .. i]:GetParent():GetID()

			-- Hide a possible glow
			ActionButton_HideOverlayGlow(_G["BagginsPooledItemButton" .. i])

			-- Hide a possible coloured border
			if(_G["BagginsPooledItemButton" .. i].wlborder ~= nil) then
				_G["BagginsPooledItemButton" .. i].wlborder:Hide()
			end

			local texture, itemCount, locked, quality, readable, lootable, itemLink = GetContainerItemInfo(parentId, id)

			if(itemLink) then
				local itemId = GetItemInfoFromLink(itemLink)
				local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice  = GetItemInfo(itemId)
				local combinedSellPrice = (itemSellPrice or 0) * itemCount

				-- If the current item is even worth less than the other one we've just found
				if(WLDB["" .. itemId] == nil and combinedSellPrice ~= 0 and quality <= WLConfig.qualityThreshold and (crappiestItemSellPrice == nil or combinedSellPrice < crappiestItemSellPrice)) then
					crappiestItemSlot = i
					crappiestItemSellPrice = combinedSellPrice
					crappiestItemQuality = quality
				end
			end
		end
	end

	-- Let it shine! (But only if an item was determined to be crap...)
	if(crappiestItemSlot ~= nil) then
		if(WLConfig.bordertype == "shine") then
			ActionButton_ShowOverlayGlow(_G["BagginsPooledItemButton" .. crappiestItemSlot])
		elseif(WLConfig.bordertype == "colour") then
			ApplyCustomBorder(_G["BagginsPooledItemButton" .. crappiestItemSlot])
		end
	end
end

--[[
	Searches the AdiBags frames for worthless items.
]]--
local function MarkAdiBags()
	local crappiestItemSlot, crappiestItemSellPrice, crappiestItemQuality

	for i = 1, 1000 do
		-- Check if the AdiBagsItemButton is in use and make sure it's not a bank slot
		if(_G["AdiBagsItemButton" .. i] ~= nil and _G["AdiBagsItemButton" .. i]:GetParent() ~= nil and _G["AdiBagsItemButton" .. i]:GetParent().isBank ~= true) then
			local id = _G["AdiBagsItemButton" .. i]:GetID()
			local parentId = _G["AdiBagsItemButton" .. i]:GetParent():GetID()

			-- Hide a possible glow
			ActionButton_HideOverlayGlow(_G["AdiBagsItemButton" .. i])

			-- Hide a possible coloured border
			if(_G["AdiBagsItemButton" .. i].wlborder ~= nil) then
				_G["AdiBagsItemButton" .. i].wlborder:Hide()
			end

			local texture, itemCount, locked, quality, readable, lootable, itemLink = GetContainerItemInfo(parentId, id)

			if(itemLink) then
				local itemId = GetItemInfoFromLink(itemLink)
				local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice  = GetItemInfo(itemId)
				local combinedSellPrice = (itemSellPrice or 0) * itemCount

				-- If the current item is even worth less than the other one we've just found
				if(WLDB["" .. itemId] == nil and combinedSellPrice ~= 0 and quality <= WLConfig.qualityThreshold and (crappiestItemSellPrice == nil or combinedSellPrice < crappiestItemSellPrice)) then
					crappiestItemSlot = i
					crappiestItemSellPrice = combinedSellPrice
					crappiestItemQuality = quality
				end
			end
		end
	end

	-- Let it shine! (But only if an item was determined to be crap...)
	if(crappiestItemSlot ~= nil) then
		if(WLConfig.bordertype == "shine") then
			ActionButton_ShowOverlayGlow(_G["AdiBagsItemButton" .. crappiestItemSlot])
		elseif(WLConfig.bordertype == "colour") then
			ApplyCustomBorder(_G["AdiBagsItemButton" .. crappiestItemSlot])
		end
	end
end

--[[
	Searches the vanilla bag frames for worthless items.
]]--
local function MarkVanilla()
	local crappiestItem_i, crappiestItem_j, crappiestItemSellPrice, crappiestItemQuality, crappiestItem_bagSize

	for i = 0, NUM_BAG_SLOTS, 1 do
		local bagSize = GetContainerNumSlots(i)
	   
		for j = 1, bagSize, 1 do
			-- Hide a possible glow
			ActionButton_HideOverlayGlow(_G["ContainerFrame" .. (i + 1) .. "Item" .. (bagSize + 1) - j])

			-- Hide a possible coloured border
			if(_G["ContainerFrame" .. (i + 1) .. "Item" .. (bagSize + 1) - j].wlborder ~= nil) then
				_G["ContainerFrame" .. (i + 1) .. "Item" .. (bagSize + 1) - j].wlborder:Hide()
			end

			local texture, itemCount, locked, quality, readable, lootable, itemLink = GetContainerItemInfo(i, j)

			if(itemLink) then
				local itemId = GetItemInfoFromLink(itemLink)
				local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice  = GetItemInfo(itemId)
				local combinedSellPrice = (itemSellPrice or 0) * itemCount

				-- If the current item is even worth less than the other one we've just found
				if(WLDB["" .. itemId] == nil and combinedSellPrice ~= 0 and quality <= WLConfig.qualityThreshold and (crappiestItemSellPrice == nil or combinedSellPrice < crappiestItemSellPrice)) then
					crappiestItem_i = i
					crappiestItem_j = j
					crappiestItemSellPrice = combinedSellPrice
					crappiestItemQuality = quality
					crappiestItem_bagSize = bagSize
				end
			end
		end
	end

	-- Let it shine! (But only if an item was determined to be crap...)
	if(crappiestItem_i ~= nil) then
		if(WLConfig.bordertype == "shine") then
			ActionButton_ShowOverlayGlow(_G["ContainerFrame" .. (crappiestItem_i + 1) .. "Item" .. (crappiestItem_bagSize + 1) - crappiestItem_j])
		elseif(WLConfig.bordertype == "colour") then
			ApplyCustomBorder(_G["ContainerFrame" .. (crappiestItem_i + 1) .. "Item" .. (crappiestItem_bagSize + 1) - crappiestItem_j])
		end
	end
end

--[[
	Searches the Bagnon/Combuctor frames for worthless items.
	Bagnon organizes items in ContainerFrameIItemJ but with larger J values than the default UI.
]]--
local function MarkBagnon()
	local crappiestItem_i, crappiestItem_j, crappiestItemSellPrice, crappiestItemQuality

	for i = 0, NUM_BAG_SLOTS, 1 do
		local bagSize = 50 -- Don't know the maximum
	   
		for j = 1, bagSize, 1 do
			if(_G["ContainerFrame" .. i .. "Item" .. j] ~= nil and _G["ContainerFrame" .. i .. "Item" .. j]:GetParent() ~= nil) then
				-- Hide a possible glow
				ActionButton_HideOverlayGlow(_G["ContainerFrame" .. i .. "Item" .. j])

				-- Hide a possible coloured border
				if(_G["ContainerFrame" .. i .. "Item" .. j].wlborder ~= nil) then
					_G["ContainerFrame" .. i .. "Item" .. j].wlborder:Hide()
				end

				local id = _G["ContainerFrame" .. i .. "Item" .. j]:GetID()
				local parentId = _G["ContainerFrame" .. i .. "Item" .. j]:GetParent():GetID()
				local texture, itemCount, locked, quality, readable, lootable, itemLink = GetContainerItemInfo(parentId, id)

				if(itemLink) then
					local itemId = GetItemInfoFromLink(itemLink)
					local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice  = GetItemInfo(itemId)
					local combinedSellPrice = (itemSellPrice or 0) * itemCount

					-- If the current item is even worth less than the other one we've just found
					if(WLDB["" .. itemId] == nil and combinedSellPrice ~= 0 and quality <= WLConfig.qualityThreshold and (crappiestItemSellPrice == nil or combinedSellPrice < crappiestItemSellPrice)) then
						crappiestItem_i = i
						crappiestItem_j = j
						crappiestItemSellPrice = combinedSellPrice
						crappiestItemQuality = quality
					end
				end
			end
		end
	end

	-- Let it shine! (But only if an item was determined to be crap...)
	if(crappiestItem_i ~= nil) then
		if(WLConfig.bordertype == "shine") then
			ActionButton_ShowOverlayGlow(_G["ContainerFrame" .. crappiestItem_i .. "Item" .. crappiestItem_j])
		elseif(WLConfig.bordertype == "colour") then
			ApplyCustomBorder(_G["ContainerFrame" .. crappiestItem_i .. "Item" .. crappiestItem_j])
		end
	end
end

--[[
	Searches the Bagnon/Combuctor frames for worthless items.
	Bagnon organizes items in ContainerFrameIItemJ but with larger J values than the default UI.
]]--
local function MarkInventorian()
	local crappiestItem_i, crappiestItem_j, crappiestItemSellPrice, crappiestItemQuality

	for i = 0, 50, 1 do -- Don't know the maximum
		local bagSize = 100 -- Don't know the maximum
	   
		for j = 1, bagSize, 1 do
			if(_G["ContainerFrame" .. i .. "Item" .. j] ~= nil and _G["ContainerFrame" .. i .. "Item" .. j]:GetParent() ~= nil) then
				-- Hide a possible glow
				ActionButton_HideOverlayGlow(_G["ContainerFrame" .. i .. "Item" .. j])

				-- Hide a possible coloured border
				if(_G["ContainerFrame" .. i .. "Item" .. j].wlborder ~= nil) then
					_G["ContainerFrame" .. i .. "Item" .. j].wlborder:Hide()
				end

				local id = _G["ContainerFrame" .. i .. "Item" .. j]:GetID()
				local parentId = _G["ContainerFrame" .. i .. "Item" .. j]:GetParent():GetID()
				local texture, itemCount, locked, quality, readable, lootable, itemLink = GetContainerItemInfo(parentId, id)

				if(itemLink) then
					local itemId = GetItemInfoFromLink(itemLink)
					local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice  = GetItemInfo(itemId)
					local combinedSellPrice = (itemSellPrice or 0) * itemCount

					-- If the current item is even worth less than the other one we've just found
					if(WLDB["" .. itemId] == nil and combinedSellPrice ~= 0 and quality <= WLConfig.qualityThreshold and (crappiestItemSellPrice == nil or combinedSellPrice < crappiestItemSellPrice)) then
						crappiestItem_i = i
						crappiestItem_j = j
						crappiestItemSellPrice = combinedSellPrice
						crappiestItemQuality = quality
					end
				end
			end
		end
	end

	-- Let it shine! (But only if an item was determined to be crap...)
	if(crappiestItem_i ~= nil) then
		if(WLConfig.bordertype == "shine") then
			ActionButton_ShowOverlayGlow(_G["ContainerFrame" .. crappiestItem_i .. "Item" .. crappiestItem_j])
		elseif(WLConfig.bordertype == "colour") then
			ApplyCustomBorder(_G["ContainerFrame" .. crappiestItem_i .. "Item" .. crappiestItem_j])
		end
	end
end

--[[
	Walks through every bag and highlights the least valuable item, also known as "crap"
	depending on installed bag addons.
]]--
function WL_MarkItemsAsCrap()
	if(WLConfig.fullMode == true and HasFreeInventorySlots()) then
		return
	end

	if(isAdiBags == true) then
		MarkAdiBags()
	elseif(isBaggins == true) then
		MarkBaggins()
	elseif(isBagnon == true or isCombuctor == true) then
		MarkBagnon()
	elseif(isInventorian == true or isTdBag2 == true) then
		MarkInventorian()
	else
		MarkVanilla()
	end
end

--[[
	Adds or removes an item id from the exclusion table.

	@param itemId

	@return If the item was added (true) or removed (false)
--]]
function WL_ManageDatabase(itemId)
	if(WLDB["" .. itemId] == nil) then
		WLDB["" .. itemId] = true

		return true
	end

	WLDB["" .. itemId] = nil

	return false
end

-- Commands
function SlashCmdList.WL(msg, editbox)
	-- No arguments, show the help.
	if(msg == "") then
		print(WL_ChatAnswer .. "Please add/remove an item via typing |cff00ff00/wl|r and shift-clicking the item or via |cff00ff00/wl {item_id}|r!")
	else
		-- Add/remove item via id
		if(string.match(msg, "^[0-9]+$")) then
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice  = GetItemInfo(msg)

			if(itemLink ~= nil) then
				if(WL_ManageDatabase(msg) == true) then
					print(WL_ChatAnswer .. "Added " .. itemLink .. " to the filter.")
					WL_MarkItemsAsCrap()
				else
					print(WL_ChatAnswer .. "Removed " .. itemLink .. " from the filter.")
					WL_MarkItemsAsCrap()
				end
			else
				print(WL_ChatAnswer .. "This item does not exist.")
			end
		-- Set the quality threshold
		elseif(string.match(msg, "^quality ([0-5])$") or string.match(msg, "^q ([0-5])$")) then
			local qualityThreshold = string.match(msg, "quality ([0-9])") or string.match(msg, "q ([0-9])")
			WLConfig.qualityThreshold = tonumber(qualityThreshold)
			print(WL_ChatAnswer .. "Now every item quality up to and including " .. qualities[qualityThreshold] .. " is taken into account.")
			WL_MarkItemsAsCrap()
		elseif(string.match(msg, "^quality .*$") or string.match(msg, "^q .*$")) then
			print(WL_ChatAnswer .. "Valid values are 0 to 5.")
		-- Output quality threshold setting
		elseif(msg == "quality" or msg == "q") then
			print(WL_ChatAnswer .. "Currently every item quality up to and including " .. qualities["" .. WLConfig.qualityThreshold] .. " is taken into account.")
		-- Set border type
		elseif(string.match(msg, "^bordertype (.*)$")) then
			local bordertype = string.match(msg, "^bordertype (.*)$")

			if(bordertype ~= "shine" and bordertype ~= "colour" and bordertype ~= "color") then
				print(WL_ChatAnswer .. "Please enter a valid border type. Valid values are |cff00ff00shine|r and |cff00ff00colour|r/|cff00ff00color|r.")
			else
				WLConfig.bordertype = bordertype

				if(bordertype == "color") then
					WLConfig.bordertype = "colour"
				end

				print(WL_ChatAnswer .. "The border type is now set to |cff00ff00" .. WLConfig.bordertype .. "|r.")
			end

			WL_MarkItemsAsCrap()
		-- Output border type
		elseif(msg == "bordertype") then
			print(WL_ChatAnswer .. "The border type is currently set to |cff00ff00" .. WLConfig.bordertype .. "|r.")
		-- Set border colour
		elseif(string.match(msg, "^bordercolour ") or string.match(msg, "^bordercolor ")) then
			local r, g, b = string.match(msg, "^bordercolour ([0-9]+),([0-9]+),([0-9]+)$")

			if(r == nil) then
				r, g, b = string.match(msg, "^bordercolor ([0-9]+),([0-9]+),([0-9]+)$")
			end

			if(r ~= nil and g ~= bil and b ~= nil and tonumber(r) >= 0 and tonumber(r) <= 255 and tonumber(g) >= 0 and tonumber(g) <= 255 and tonumber(b) >= 0 and tonumber(b) <= 255) then
				WLConfig.borderColourR = tonumber(r)
				WLConfig.borderColourG = tonumber(g)
				WLConfig.borderColourB = tonumber(b)
				WL_MarkItemsAsCrap()
				print(WL_ChatAnswer .. "The border colour is now set to |cff00ff00" .. r .. "," .. g .. "," .. b .. "|r.")
			else
				print(WL_ChatAnswer .. "Usage: |cff00ff00/wl bordercolour [0-255],[0-255],[0-255]|r.")
			end
		-- Output border colour
		elseif(msg == "bordercolour" or msg == "bordercolor") then
				print(WL_ChatAnswer .. "The border colour is currently set to |cff00ff00" .. WLConfig.borderColourR .. "," .. WLConfig.borderColourG .. "," .. WLConfig.borderColourB .. "|r.")
		-- Full Mode true
		elseif(msg == "onlyfull true") then
			WLConfig.fullMode = true
			WL_MarkItemsAsCrap()
			print(WL_ChatAnswer .. "From now on items will only get highlighted when your inventory is full.")
		-- Full Mode false
		elseif(msg == "onlyfull false") then
			WLConfig.fullMode = false
			WL_MarkItemsAsCrap()
			print(WL_ChatAnswer .. "From now on items will also get highlighted when your inventory is not full.")
		-- Output Full Mode
		elseif(msg == "onlyfull") then
			if(WLConfig.fullMode == true) then
				print(WL_ChatAnswer .. "Currently items will only get highlighted when your inventory is full.")
			else
				print(WL_ChatAnswer .. "Currently items will also get highlighted when your inventory is not full.")
			end
		-- Clear database
		elseif(msg == "clear") then
			WLDB = {}
			print(WL_ChatAnswer .. "Database cleared.")
		-- Add/remove item via item link posting
		elseif(GetItemInfoFromLink(msg) ~= nil) then
			local itemId = GetItemInfoFromLink(msg)
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice  = GetItemInfo(itemId)

			if(itemLink ~= nil) then
				if(WL_ManageDatabase(itemId) == true) then
					print(WL_ChatAnswer .. "Added " .. msg .. " to the filter.")
					WL_MarkItemsAsCrap()
				else
					print(WL_ChatAnswer .. "Removed " .. msg .. " from the filter.")
					WL_MarkItemsAsCrap()
				end
			else
				print(WL_ChatAnswer .. "This item does not exist.")
			end
		else
			print(WL_ChatAnswer .. "This command does not exist.")
		end
	end
end